library ImageIODemo;

{This is a sample DLL project that demonstrates how an implementation of a DLL could look like
that utilizes the X-Ways Forensics Image I/O API. It does not do any actual image file I/O.
Instead, it just pretends to understand all kinds of files and when requested to read from them
it fills the supplied buffer with some fake data.}

uses Windows;

type
	PIIOImageInfo=^TIIOImageInfo;
	TIIOImageInfo=record
		nSize: DWord;
		nSectorCount: Int64;
		nSectorSize: DWord;
		nFlags: DWord;
		lpTextualDescr: PWideChar;
	end;

	PMyImageInfo=^TMyImageInfo;
	TMyImageInfo=record
		{Our own private structure.}
		FileHdl: THandle;
		SegmentCount: DWord;
		VariousOtherData: Pointer;
	end;

const
	IIO_INIT_READ = $0001; // return this to signal read support for the image
	IIO_INIT_WRITE = $0002; // return this to signal write support for the image
	IIO_INIT_DISK = $0010; // image represents a physical storage device, possibly partitioned
	IIO_INIT_VOLUME = $0020; // image represents a volume/partition
	IIO_INIT_THREADSAFE = $0100; // your functions are thread-safe (currently a must)
	IIO_INIT_UNALIGNED_OK = $0200; // your functions support sector-unaligned I/O requests (not a must)
	IIO_INIT_ERROR_MILD = $1000; // errors in the image have been detected, but it can presumable be read completely
	IIO_INIT_ERROR_SEVERE = $2000; // errors in the image have been detected, it cannot be fully read
	IIO_INIT_ERROR_GIVE_UP = $4000; // image type generally supported, but unable to open this particular file, lpTextualDescr points to an error message

var
	MainWnd: HWnd;

function StrRScanW(Str: PWideChar; Chr: WideChar): PWideChar;
var
	Loop, Len: Word;
begin
	Result:=nil;

	Len:=lStrLenW(Str);

	if Len>0 then
	for Loop:=Len-1 downto 0 do begin
		if (Str+Loop)^=Chr then begin
			Result:=Str+Loop;
			break;
		end;
	end;
end;

function GetFileNameFromPathW(FilePath: PWideChar): PWideChar;
var
	P: PWideChar;
begin
	P:=StrRScanW(FilePath, '\');
	if P=nil then Result:=FilePath
	else Result:=P+1;
end;

function IIO_Init(hMainWnd: HWnd; lpFilePath: PWideChar; pHeaderBuf: Pointer; nHeaderBufSize: DWord; pImgInfo: PIIOImageInfo; pReserved: Pointer): PMyImageInfo; stdcall;
const
	ExpectedSignature=123456789;
begin
	if (PDWord(pHeaderBuf)^=ExpectedSignature)
	or true {this demo DLL feels responsible for EVERY file that X-Ways Forensics tries to interpret}
	then begin
		MessageBoxW(hMainWnd, 'Ready to simulate image I/O for this image.', GetFileNameFromPathW(lpFilePath), MB_ICONINFORMATION);
		MainWnd:=hMainWnd; //remember for later

		pImgInfo.nSize:=SizeOf(TIIOImageInfo);
		pImgInfo.nSectorCount:=1024*1024;
		pImgInfo.nSectorSize:=1024;
		pImgInfo.nFlags:=IIO_INIT_READ or IIO_INIT_DISK;

		GetMem(pImgInfo.lpTextualDescr, 128);
		lStrCpyW(pImgInfo.lpTextualDescr, 'Just a demo DLL at work. Don''t trust this data, OK?');

		GetMem(Result, SizeOf(TMyImageInfo));
		Result.SegmentCount:=1;
	end else Result:=nil;
end;

function IIO_Work(lpImage: Pointer; nOfs, nSize: Int64; lpBuffer: Pointer; pFlags: PDWord): Int64; stdcall;
begin
	FillChar(lpBuffer^, nSize, $33);

	if (nOfs mod 512=0) and (nSize>=16) then
		CopyMemory(lpBuffer, PAnsiChar('FAKEDATAFAKEDATA'), 16);

	Result:=nSize; // signal total success :)
end;

function IIO_Done(lpImage: PMyImageInfo; lpTextualDescr: PWideChar): DWord; stdcall;
begin
	{Clean up memory buffers.}
	FreeMem(lpTextualDescr);
	FreeMem(lpImage);
	MessageBoxA(MainWnd, 'Quite exhausting to do so much image I/O work! Taking a rest now.', 'Done', MB_ICONINFORMATION);
end;

exports
	IIO_Init,
	IIO_Work,
	IIO_Done;

begin
	{Here you could insert code that gets executed when the DLL is loaded.}

end.
